use Time::Local;


my(%log_levels) = ( "ERROR" => 0,
                        "WARNING" => 1,
                        "STATUS" => 2,
                        "PROFILE" => 3,
                        "VERBOSE" => 4,
                        "DEBUG" => 5);

# List the files, most recent file first (based on the filename)
# which has the timestamp encoded within it
my($LOGFILE_LOCATION) = "/var/local/filepool/Logfiles";
my($fileNamesString);
getLogFiles($LOGFILE_LOCATION);

sub getLogFiles {
        my $directory = shift;
        if (-d $directory)
        {
                local *LOGFILES;
                my $result = opendir(LOGFILES, $directory);
                if ($result)
                {
                        my @files = readdir(LOGFILES);
                        foreach $file (@files)
                        {
                                if (($file ne ".") && ($file ne ".."))
                                {
                                        if (-d $directory . "/" . $file)
                                        {
                                                getLogFiles($directory . "/" . $file);
                                        } else {
                                                if ($file =~ m/FilePool.*/g)
                                                {
                                                        $fileNamesString = $fileNamesString . $directory . "/" . $file . " ";
                                                }
                                        }
                                }
                        }
                        closedir(LOGFILES);
                }
        }
}

my(@files) = split / /, $fileNamesString;
my($file);

# Get the filter level, defaulting to ERROR, and optionally
# supplied by the client (by prepending a line setting $argument).
my($filter_level) = $log_levels{"ERROR"};

if ($argument ne "") {
    if (defined($log_levels{$argument})) {
        $filter_level = $log_levels{$argument};
    } else {
        print "Unknown event level $argument.\n";
        exit 1;
    }
}

my($MAX_RETURNED_LOG_ENTRIES) = 500;
my(@log_entries) = ();
my($truncated) = 0;
foreach $file (@files)
    {
    my(@local_log_entries) = ();

    # Each file contains at most 24 hours worth of log entries, but the log entries may span
    # a day.  Individual log entries don't have the day, only the time, so here we get the
    # year/month/day to attach to the individual log entries.
    # However, the day (and possibly the month/year) can change within a single log file,
    # so here we check for that and handle it (if the time within the day of a log entry is
    # less than the time within the day of the logfile start time, then the entry must be for the
    # next day!)
    my($position_of_level) = -1;                                # position of severity level in logfiles
    my($filename_time_string) = $file;                          #
    $filename_time_string =~ s/^.*FilePool_Log_//;              # Remove leading characters
    $filename_time_string =~ s/\.log.*$//;                      # Remove trailing characters
    $filename_time_string =~ s/.*?-.*?-.*?-.*?_(.*)/$1/;        # Remove version if present
    my($day_spec, $time_spec, $tz) = split(/_/, $filename_time_string);
    my($year, $month, $mday) = split(/\./, $day_spec);
    my($hour, $min, $sec) = split(/\./, $time_spec);

    my($start_time_value) = timegm($sec, $min, $hour, $mday, $month - 1, $year - 1900);

    if ($file =~ /.gz$/) {
        next if (!open(LOGFILE, "zcat $file |"));
    } else {
        next if (!open(LOGFILE, "<$file"));
    }

    # Process the logfile, only keeping entries which match the filter criteria
    while (<LOGFILE>) {
        my ($line) = $_;
        if ($position_of_level < 0) {
            #
            # try to find out where the type field is located
            #
            chomp;
            my(@header)= split('\t', $line);
            if ($header[0] eq 'Timestamp') {
                for ($i=0; $i<=$#header; $i++) {
                    if ($header[$i] eq 'Type') {
                        $position_of_level = $i;
                    }
                }
            }
        }
        next if (!/^[0-9]/); # skip non-log entry lines

        chomp($line);
        my(@splitted) = split('\t', $line);

        my ($logentry) = '';
        if ($log_levels{$splitted[$position_of_level]} <= $filter_level) {
            my($entry_hour, $entry_min, $entry_sec, $entry_ms);
            my($entry_time_value);

            $entry_hour = -1;
            if ($splitted[0] =~ /^\d\d\.\d\d\.\d\d\.\d\d\d/) {
                # old format - just time (hour.min.sec.milliseconds)
                ($entry_hour, $entry_min, $entry_sec, $entry_ms) = split(/\./, $splitted[0]);
            } elsif ($splitted[0] =~ /^\d\d\d\d.\d\d.\d\d (\d\d):(\d\d):(\d\d).(\d+)/) {
                # new format - year.month.day hour:min:sec
                $entry_hour = $1;
                $entry_min = $2;
                $entry_sec = $3;
                $entry_ms = $4;
            } else {
                # unknown format - just leave it alone.
            }
            if ($entry_hour >= 0) {
                $entry_time_value = timegm($entry_sec, $entry_min, $entry_hour, $mday, $month - 1, $year - 1900);
                # Did the entries in the log file move onto the next day?
                if ($entry_time_value < $start_time_value) {
                    $entry_time_value += 3600 * 24; # yes, add a day
                    ($sec, $min, $hour, $mday, $month, $year) = gmtime($entry_time_value);
                    $year += 1900;
                    $month++;
                    $start_time_value = $entry_time_value;
                }
                $log_entry = sprintf "%04d.%02d.%02d_%02d.%02d.%02d.%03d_%s", $year, $month, $mday, $entry_hour, $entry_min, $entry_sec, $entry_ms, $tz;
                for ($i=1; $i<=$#splitted; $i++) {
                    $log_entry = $log_entry . "    ${splitted[$i]}";
                }
                unshift(@local_log_entries, $log_entry);
            }
        }
    }
    close(LOGFILE);

    # Add these entries after the entries from other log files (as they were chronologically earlier)
    push(@log_entries, @local_log_entries);

    if ($#log_entries >= $MAX_RETURNED_LOG_ENTRIES) {
        $truncated = 1;
        last;
    }
}

my($index) = 0;
my($end_index) = $#log_entries + 1;
if ($truncated) {
    print "Event list truncated at $MAX_RETURNED_LOG_ENTRIES entries.\n\n";
    $end_index = $MAX_RETURNED_LOG_ENTRIES;
}

for ($index = 0; $index <= $end_index; $index++) {
    print $log_entries[$index], "\n";
}

exit(0);
