#
# Get and set device states at the OS level
# Parameters:
#  oscommand
#	fpgetdevices - return a list of devices and their states
#	fpsetdevicestate - set the state of a device
#
# The following are only for fpsetdevicestate
#  type - the type of device (nic or disk)
#  state - the new state of the device (enabled or disabled)
#  name - the name of the device (eth0, hdf10, etc.)
#
#
# Information is returned as XML in the following form
#
# <command status="OK/FAIL" />
# <error value="xxx"/> for errors
# <device name="xxx" type="xxx" state="xxx"/>  multiple of these
#
# 01/02/2002 Mark O'Connell     created
# 17/05/2004 Ivo Abeloos        TT12934, removed mode check
# 02/07/2004 Ivo Abeloos        TT13591, replace ifconfig up/down with ifup/ifdown
# 04/03/2005 Ivo Abeloos        TT13143, disk/volume inconsistencies
# 16/03/2005 Ivo Abeloos        also stop/start fragmentcounter
# 14-03-2005 Ivo Abeloos        order of disks is important
# 15-03-2005 Ivo Abeloos        typo + NIC probs TT18590
# 19-03-2005 Ivo Abeloos        use revivedisk where available
# 02-05-2005 Ivo Abeloos        use special CV report
# 07-10-2005 Ivo Abeloos        new server jar
#

use File::stat;

my %cmdparams;
my $p;
my $debug = 0;

# Get parameters
foreach $p (@args) {
    my ( $k, $v ) = split /=/, $p;
    $cmdparams{$k} = $v;
}

# By default, stderr is connected to the
# client socket.  Make sure that writes to
# stderr don't accidentally make it back to the
# client.
close(STDERR);
open( STDERR, ">/dev/null" );

if ($debug) {
    open( DEBUG, ">>/tmp/fpdevice.debug" );
    print DEBUG "$$: Starting at..." . qx(date);
    my ($cmd) = "ls -l /proc/$$/fd";
    print DEBUG "$$: My fds are ($cmd): \n" . qx($cmd);
}

########################################################
# Return NIC status information to the caller.
########################################################
my ($NIC_NAME)  = "nic_name";
my ($NIC_STATE) = "nic_state";
my ($EDMROOT)   = "/var/local/edm";

sub get_nic_status {

    #
    # NIC part
    #
    # Get the nic list and state (from ifconfig)
    if ( !open( NIC_INFO_PIPE, "/sbin/ifconfig -a |" ) ) {
        return ();
    }

    my ( $nic_name, $nic_state );
    my ($nic_return_string) = "";

    my (@nic_info);
    while (<NIC_INFO_PIPE>) {
        ($nic_name) = split;
        $_ = <NIC_INFO_PIPE>;    # Skip the second line
        $_ = <NIC_INFO_PIPE>;    # Third line has the flags
        if (/ UP /) {            # Look for the UP and NOARP flag
            $nic_state = "enabled";
            if (/NOARP /) {
                $nic_state = "disabled";
            }
        } else {
            $nic_state = "disabled";
        }

        while (<NIC_INFO_PIPE>) {    # Skip rest of this NIC
            last if (/^[ 	]*$/);     # NIC ends at an empty line
        }

        if ( $nic_name !~ /^lo/ ) {
            my ($nic_device) = {};
            $nic_device->{$NIC_NAME}  = $nic_name;
            $nic_device->{$NIC_STATE} = $nic_state;
            push( @nic_info, $nic_device );
        }
    }

    close(NIC_INFO_PIPE);

    return (@nic_info);
}

########################################################
# Return Disk status information to the caller.
########################################################
my ($DISK_NAME) = "disk_name";
my ($DISK_STATE)= "disk_state";

my ($CONFIG_DIR) = "/var/local/bootstat";

my ($SHORT_NAME) = "short_name";
my ($DEV_NAME)   = "dev_names";
my ($FOUND)      = "found";
my ($MOUNTPOINT) = "mountpoint";
my ($NUMBER)     = "number";
my ($FULLNAME)   = "fullname";
my ($STATE)      = "state";

sub get_old_disk_status {
    my (@drives);
    my ($drive_info);

    # Pull information about disk configuration.
    # See /etc/rc.d/rc.sysinit - it also does this to
    # determine filesystem mountpoints
    @drives = qx(ls -d $CONFIG_DIR/DISK-\?-STATUS);
    map { s#$CONFIG_DIR/## } @drives;
    map { s#^DISK-(\w)-STATUS$#$1# } @drives;
    map { chomp($_) } @drives;

    my ( $dummy, $blob_partition ) =
      split( ',', qx(grep "^blobs," "$CONFIG_DIR/PARTITIONS") );
    my ($disk_number) = 0;
    my ($drive);
    my (@full_drive_names);
    foreach $drive (@drives) {
        $disk_number++;

        my ($realname) = {};
        $realname->{$SHORT_NAME} = "hd$drive$blob_partition";
        $realname->{$DEV_NAME}   = "/dev/" . $realname->{$SHORT_NAME};
        $realname->{$DISK_NAME}  = "/dev/" . $realname->{$SHORT_NAME};
        $realname->{$NUMBER}     = $disk_number;
        $realname->{$MOUNTPOINT} = "/mnt/blobs$disk_number";
        if ( -l $realname->{$DEV_NAME} ) {
            $realname->{$FULLNAME} = "/dev/" . readlink $realname->{$DEV_NAME};
        }

        push( @full_drive_names, $realname );
    }

    # See where the filesystems are actually mounted.
    # This is used to determine the disk state.
    if ( !open( MOUNT_PIPE, "mount |" ) ) {
        return ();
    }

    my (%mount_hash);
    while (<MOUNT_PIPE>) {
        my ( $device, $dummy, $mountpoint ) = split;
        $mount_hash{$device} = $mountpoint;
    }

    close(MOUNT_PIPE);

    # See if the drive is mounted or not.
    # If mounted, check to see that the mount is
    # correct.
    foreach $drive_info (@full_drive_names) {
        my ($mountpoint) = "";
        $mountpoint = $mount_hash{ $drive_info->{$DEV_NAME} }
          if ( defined( $mount_hash{ $drive_info->{$DEV_NAME} } ) );
        if ( $mountpoint eq "" ) {
            $mountpoint = $mount_hash{ $drive_info->{$FULLNAME} }
              if ( defined( $drive_info->{$FULLNAME} )
                && defined( $mount_hash{ $drive_info->{$FULLNAME} } ) );
        }

        my ($state);

        if ( $mountpoint ne "" )

        # Check mount permissions and ownership
        {
            $stats = stat($mountpoint);

            $uid  = $stats->uid;
            $gid  = $stats->gid;
            $mode = $stats->mode;
            if ( ( $uid eq "500" ) && ( $gid eq "500" ) ) {
                $mountcheck = "ok";
                print "MC = OK\n";
            }
            else {
                $mountcheck = "nok";
                print "MC = NOT OK\n";
            }

            if (   ( $mountpoint eq $drive_info->{$MOUNTPOINT} )
                && ( $mountcheck eq "ok" ) )
            {
                $state = "enabled";
            }
            else {
                $state = "wrong_mount [$uid,$gid]";
            }
        }
        else {
            $state = "disabled";
        }
        $drive_info->{$STATE} = $state;
        $drive_info->{$DISK_STATE} = $state;
    }

    return (@full_drive_names);
}    # get_old_disk_status

sub get_disk_status {
    my (@drives);
    my ($drive_info);

    $command="export FILEPOOL_HOME=/home/filepool/FP/Lib;";
    $command=$command . "export LD_LIBRARY_PATH=\$FILEPOOL_HOME;";
    $command=$command . "export JAVA=`cat /home/filepool/FP/java`/bin/java;";
    $command=$command . "\$JAVA -cp \$FILEPOOL_HOME/commons-lang-2.0.jar:\$FILEPOOL_HOME/commons-lang-2.1-rc1.jar:\$FILEPOOL_HOME/commons-lang-2.1.jar:";
    $command=$command . "\$FILEPOOL_HOME/Filepool.zip:\$FILEPOOL_HOME/platforminterfaces.jar:\$FILEPOOL_HOME/platformapi.jar ";
    $command=$command . "com.filepool.platformsupport.report.PlatformAPIReportForCV";
    $platformAPI = `sh -c \'$command\'`;
    print DEBUG "$$: platform API returned: " . $platformAPI . "\n" if ($debug);
    my @disk_info=($platformAPI =~ /(\/dev\S+)\s+(\S+)/igs);

    my ($drive);
    my (@full_drive_names);
    for ($i=0; $i<=$#disk_info; $i=$i+2) {
        my ($realname) = {};
        $realname->{$DISK_NAME} = $disk_info[$i] . ":" . ($i / 2);
        $realname->{$DISK_STATE} = "disabled";
        if (lc($disk_info[$i+1] eq "true")) {
            $realname->{$DISK_STATE} = "enabled";
        }
        push(@full_drive_names, $realname);
    }
    return (@full_drive_names);
}    # get_disk_status

#
# deref, tries to get to the symbolic link of the disk name
#
sub deref {
    my ($disk) = $_[0];
    my ($name);
    if (-l $disk) {
        my($dirname) = `dirname $disk`;
	    chomp($dirname);
        my($pattern) = "s/^.* -> \\\(.*\\\)\\\$/\\1/p";
        my($linked) = `ls -l $disk | sed -ne"${pattern}"`;
	    chomp($linked);
        $name = "${dirname}/${linked}";
    } else {
        $name = $disk;
    }
    chomp($name);
    return $name;
}

########################################################
# Set a nic or disk device to enabled/disabled
########################################################
sub set_device_state {
    my ($rval);
    my (@drives) = @{ $_[0] };
    my (@nics)   = @{ $_[1] };
    my (@cmds);

    if (   ( $cmdparams{"state"} ne "enabled" )
        && ( $cmdparams{"state"} ne "disabled" ) )
    {
        return ("unknown device state");
    }

    print DEBUG "$$: Command "
      . $cmdparams{"state"} . " for "
      . $cmdparams{"type"}
      . " named "
      . $cmdparams{"name"} . "\n\n"
      if ($debug);

    my ($set_to_enabled) = ( $cmdparams{"state"} eq "enabled" );

    if ( $cmdparams{"type"} eq "nic" ) {
        print DEBUG "$$: Setting NIC device.\n" if ($debug);
        my ($nic_info);
        my ($found) = 0;
        foreach $nic_info (@nics) {
            if ( $nic_info->{$NIC_NAME} eq $cmdparams{"name"} ) {
                print DEBUG "$$: Status of "
                  . $nic_info->{$NIC_NAME} . " is "
                  . $nic_info->{$NIC_STATE} . "\n"
                  if ($debug);
                if ($set_to_enabled) {
                    if ( $nic_info->{$NIC_STATE} eq "enabled" ) {
                    }
                    else {
                        $nic_info->{$NIC_STATE} = "enabling";
                        push( @cmds,
                            "rm /tmp/" . uc( $cmdparams{"name"} ) . "_DOWN" );
                        push( @cmds, "ifup " . $cmdparams{"name"});
                    }
                }
                else {
                    if ( $nic_info->{$NIC_STATE} eq "disabled" ) {
                    }
                    else {
                        $nic_info->{$NIC_STATE} = "disabling";
                        push( @cmds, "ifdown " . $cmdparams{"name"});
                        push( @cmds,
                                "touch /tmp/"
                              . uc( $cmdparams{"name"} )
                              . "_DOWN" );
                    }
                }

                $found = 1;
                last;
            }
        }

        if ( !$found ) {
            return ("no such device");
        }
    } elsif ( $cmdparams{"type"} eq "disk" ) {
        print DEBUG "$$: Setting disk device.\n"    if ($debug);
        print DEBUG "$$: Got status on $#drives.\n" if ($debug);
        if ( $#drives == -1 ) {
            return ("cannot set disk status");
        }

        my ($found) = 0;
        my ($drive_info);
        foreach $drive_info (@drives) {
            if ( $drive_info->{$DISK_NAME} eq $cmdparams{"name"} ) {
                print DEBUG "$$: Status of "
                  . $drive_info->{$DISK_NAME} . " is "
                  . $drive_info->{$DISK_STATE} . "\n"
                  if ($debug);
                if ($set_to_enabled) {
                    if ( $drive_info->{$DISK_STATE} eq "enabled" ) {
                    } else {
                        $drive_info->{$DISK_STATE} = "enabling";
                        push( @cmds, "service filepool stop" );
                        push( @cmds, "service fragmentcounter stop" );
                        #
                        # is the revivedisk script available ? if so, make sure
                        # the disk is marked OK
                        #
                        my ($disk_short, $disk_index) = split /:/, $drive_info->{$DISK_NAME};
                        my($revive) = ( -e "/sbin/FPrevivedisk" );
                        if ($revive) {
                            my($real) = deref($disk_short);
                            my($command) = "smartctl -a ${real} | grep \"^Serial\" | grep -o '[A-z0-9]*\\W*\$' | sed -e '{s/ //g}'";
                            my($serial) = `${command}`;
			                chomp($serial);
                            my($command) = "cat ${EDMROOT}/disk/${serial}/status 2>/dev/null";
                            my($status) = `${command}`;
			                chomp($status);
                            if ($status ne "OK") {
                                #
                                # replace status file
                                #
                                my($command) = "rm -f ${EDMROOT}/disk/${serial}/* ";
                                my($result) = `${command}`;
                                my($command) = "echo \"OK\" >${EDMROOT}/disk/${serial}/status";
                                my($result) = `${command}`;
                            }
                        }
                        if ( $cmdparams{"format"} eq "true" ) {
                            push( @cmds,
                                "yes | mkreiserfs "
                                  . $disk_short );
                        }
                        push( @cmds, "service blobpartitions restart" );
                        push( @cmds, "service fragmentcounter start" );
                        push( @cmds, "service filepool start" );
                    }
                }
                else {
                    if ( $drive_info->{$DISK_STATE} eq "disabled" ) {
                    }
                    else {
                        my ($disk_short, $disk_index) = split /:/, $drive_info->{$DISK_NAME};
                        $drive_info->{$DISK_STATE} = "disabling";
                        push( @cmds, "service filepool stop" );
                        push( @cmds, "service fragmentcounter stop" );
                        push( @cmds, "service blobpartitions faildisk " .
                            $disk_short);
                        push( @cmds, "service fragmentcounter start" );
                        push( @cmds, "service filepool start" );
                    }
                }

                $found = 1;
                last;
            }
        }

        if ( !$found ) {
            return ("no such device");
        }
    }
    else {
        return ("unknown device type");
    }

    if ( ( $#cmds != -1 ) && ( fork() == 0 ) ) {
        close(STDOUT);
        open( STDOUT, ">/dev/null" );

        sleep 5;
        my ($cmd);
        foreach $cmd (@cmds) {
            system($cmd);
        }
        print DEBUG "$$: Did the cmds " . join( "\n\t", @cmds ) . "\n"
          if ($debug);
        exit 0;
    }

    return ("");
}

########################################################
# Mainline code
########################################################
if ( !defined( $cmdparams{"oscommand"} ) ) {
    print "<command status=\"FAIL\"/>\n";
    print "<error value=\"unknown command\"/>\n";

    print DEBUG "$$: unknown command...\n" if ($debug);
    close(DEBUG)                           if ($debug);

    exit 0;
}

my($faro) = ( -e "/usr/local/bin/platformAPIReport.sh" );
my (@drives);
if ($faro) {
    @drives = &get_disk_status;
} else {
    @drives = &get_old_disk_status;
}
my (@nics)   = &get_nic_status;

my ($set_error) = "";
if ( $cmdparams{"oscommand"} eq "fpsetdevicestate" ) {
    $set_error = &set_device_state( \@drives, \@nics );
}

if ( ( $#drives == -1 ) || ( $#nics == -1 ) || ( $set_error ne "" ) ) {
    print "<command status=\"FAIL\"/>\n";
    if ( $set_error eq "" ) {
        print "<error value=\"unable to retrieve device list\"/>\n";
        print DEBUG "$$: Unable to retrieve device list.\n"     if ($debug);
        print DEBUG "$$: Drives is $#drives, nics is $#nics.\n" if ($debug);
    }
    else {
        print "<error value=\"$set_error\"/>\n";
        print DEBUG
          "$$: Drives is $#drives, nics is $#nics, set error is $set_error.\n"
          if ($debug);
    }
}
else {
    print "<command status=\"OK\"/>\n";
}

my ($output) = "";
my ($drive_info);
foreach $drive_info (@drives) {
    $output .=
        "<device type=\"disk\" name=\""
      . $drive_info->{$DISK_NAME}
      . "\" state=\""
      . $drive_info->{$DISK_STATE}
      . "\"/>\n";
}

my ($nic_info);
foreach $nic_info (@nics) {
    $output .=
        "<device type=\"nic\" name=\""
      . $nic_info->{$NIC_NAME}
      . "\" state=\""
      . $nic_info->{$NIC_STATE}
      . "\"/>\n";
}

print DEBUG "$$: output is $output.\n" if ($debug);
close(DEBUG)                           if ($debug);

print $output;
exit 0;
