################################################################
# log_process.pl
# 
# $pre_process := "YES" or "NO" 
#			(default NO)
# $LOGFILE_LOCATION := /var/log or some other directory 
#			(default /var/local/filepool/Logfiles)
# @log_list := list of logs from $LOGFILE_LOCATION to download
#			(default *)
#

#######################################################
#Global variables 
#######################################################

#Global variables to share between parent and child procesees
our ($download_dir)          = "";    # new log directory  to process log files
our ($child_id)              = 0;     # child pid
our ($parent_success)        = "NO"; # is this daemonizing process successful/not 
our ($parent_failure_reason) = ""; # reason for failure if so.

# constants
our ($NODE_NAME)          = `hostname`; # host name for this node
$NODE_NAME =~ s/\n//;

our ($CVLOG_DIR_PREFIX)   = "CVDWNLD_";# prefix text to log directory
our ($TEMP_DIR)           = "/tmp/.centeraviewer";     # temporary directory of linux sys

#check if temporary directory exists, if not create
if (!(-e "/tmp/.centeraviewer"))
{
    mkdir("/tmp/.centeraviewer");
    chmod 0777, "/tmp/.centeraviewer";
}

#default values
if (!defined(${LOGFILE_LOCATION}))
{
  ${LOGFILE_LOCATION}   = "/var/local/filepool/Logfiles";
}

# Initialize log_list, we do this in case @log_list is not defined
# or if @log_list is *

if (($log_list[0] =~ '\*') ||
    (!defined(@log_list)))
{
 chdir $LOGFILE_LOCATION;
 @log_list = qx(ls -1 * 2>/dev/null);
 chomp(@log_list);
}

if (!defined($pre_process))
{
  $pre_process = "NO";
}

##########################################################
# function to fork the child.                            #
# If fork fails with recoverable problems,it tries again.#
##########################################################
sub FORK
{
   if ($pid = fork) 
   {
        # parent
        parent_process();
   } 
   elsif (defined $pid) { # $pid is zero here if defined
        # child
        child_process();
   } elsif ($! =~ /No more process/) {
        # Try Again, supposedly recoverable fork error
        sleep 5;
        redo FORK;
   } else {
        # fork operation failed.
        $parent_failure_reason = "Fork Operation Failed";
        print_output();
        exit 0;
   }
}

#########################################################
# Function to fork and daemonize child process.         #
# This function also decides on download directory.     #
#########################################################
sub daemonize_child
{
   my ($currenttime) = time;
   $download_dir = "$CVLOG_DIR_PREFIX${NODE_NAME}_$currenttime";
   
   # fork the process.
   FORK();
}

####################################################
#functions related to parent process
####################################################

# parent process. 
sub parent_process
{
 $child_id = $pid;
 $parent_success = "YES";
 print_output();

# close handles so we actually exit after fork()

 close(STDIN);
 close(STDOUT);
 close(STDERR);

 exit 0;
}

#################################################
# print main process output in xml format. 
#################################################
sub print_output
{
 print "<output>\n";
 print "<node value = $NODE_NAME/>\n";
 print "<success value = $parent_success/>\n";

 if($parent_success =~  /NO/)
 {
   print "<failure_reason value = \"$parent_failure_reason\"/>\n"; 
 }

 print "<child_id value = $child_id/>\n";
 print "<download_dir value = $download_dir/> \n";
 print "</output>\n";
}

sub preprocess_logs
{

my(@rules) = ( [ "MD5VerifierTask", "^TaskFailed" ],
		[ "Update Log Processing", "^TaskStarted" ],
		[ "Update Log Processing", "^TaskStopped" ],
		[ "OrganicRegeneratorTask", "^TaskStarted" ],
		[ "OrganicRegeneratorTask", "^TaskStopped" ],
		[ "EventStreamers", "UnknownStatistic" ],
		[ "Cluster Handlers", "NODE_PARAMSCHANGE.*\\\(online\\\)\$" ] );


my($current_time) = time;

my($preprocessed_logs_dir) = $TEMP_DIR . "/" . $download_dir;

my ($file);
chdir "$LOGFILE_LOCATION";

foreach $file (@log_list)
    {
    my($outfile_name) = $file;
    $outfile_name =~ s#^$LOGFILE_LOCATION##;
	# Strip off everything but the base filename, like qx(basename)

    if ($file =~ /\.gz$/)
	{
	next if (!open(LOGFILE, "zcat $file |"));
	$outfile_name =~ s/\.gz//;	# Strip off the suffix indicating gzip compression
	}
    else
	{
	next if (!open(LOGFILE, "<$file"));
	}

    if (!open(OUTFILE, ">$preprocessed_logs_dir/$outfile_name"))
	{
	close(LOGFILE);
	next;
	}

    # Process the logfile, only keeping entries which match the filter criteria
    while (<LOGFILE>)
	{
	my($include) = 1;
	# Only exclude lines which are standard log entry lines
	if (/^[0-9]/)
	    {
	    split('\t');

	    my($rule);
	    foreach $rule (@rules)
		{
		# $_[1] holds the thread name, $_[5] the log message
		if (($_[1] =~ /$rule->[0]/) && ($_[5] =~ /$rule->[1]/))
		    {
		    $include = 0;
		    last;
		    }
		}
	    }

	if ($include)
	    {
	    print OUTFILE;
	    }
	}

    close(LOGFILE);
    close(OUTFILE);
    }
}


#################################################################
#functions related to child process                             #
# child is deamonized by parent process and parent terminates   #
# child pid is reported back to cv for checking it's completion.#
# child process does not print / report any problems.           #
#################################################################

#child process
sub child_process
{ 
 open(STDIN,"</dev/null");
 open(STDOUT,">/dev/null");
 open(STDERR,">/dev/null");

 delete_old_download_dirs();

 if (!mkdir("$TEMP_DIR/$download_dir"))
 {
     exit 0;
 }

 if ($pre_process =~ /YES/)
 {
     preprocess_logs();
     chdir "$TEMP_DIR/$download_dir";
 }
 else
 {
     chdir "$LOGFILE_LOCATION";
 }

 my ($tarfile) = "$TEMP_DIR/$download_dir/$download_dir." . "tar.gz";

 if ($pre_process =~ /YES/)
 {
  # just tar the whole directory
  qx ( tar -zcvf $tarfile . > /dev/null 2>&1 );
 }
 else
 {
  qx ( tar -zcvf $tarfile @log_list > /dev/null 2>&1 );
 }

 cleanup_preprocessed();

 exit 0;
}

sub cleanup_preprocessed
{
    chdir "$TEMP_DIR/$download_dir";
    @temp_files = qx(ls -1 $TEMP_DIR/$download_dir/* 2> /dev/null);
    chomp(@temp_files);

    foreach $file (@temp_files)
    {
	unless ($file =~ /\.tar\.gz$/)
        {
	    unlink($file);
        }
    }
}

#delete tmp directories created by download process, more than an hour ago.
sub delete_old_download_dirs
{
    my($TIME_LIMIT) = 3600;
    my($current_time) = time;

    chdir "$TEMP_DIR";
    # Remove any old entries in tmp (more than an hour old is too old)
    my(@old_entries) = qx(ls -d -1 $TEMP_DIR/$CVLOG_DIR_PREFIX* 2>/dev/null);
    chomp(@old_entries);
    my($old_entry);
    foreach $old_entry (@old_entries)
    {
      my($prefix, $nodename, $entry_time) = split(/_/, $old_entry);
      if (($current_time - $entry_time) > $TIME_LIMIT)
      {
        system("rm -rf $old_entry > /dev/null  2>&1");
	#print "would have rm -rf'd $old_entry\n";
      }
    }
 }

####################################################
# MAIN PROCESS
####################################################
# process inputs from CV to get options to preprocess, log list
# decide the name of the download directory daemonize the child 
# report download directory, child process id and terminate parent process. 
# child process does the main task of preprocessing and compressing logs
# child process terminates after operation is completed. 
# CV keeps on checking for pid of child to check completion.

daemonize_child();  #daemonize the child process

$parent_failure_reason = "Problem in forking";
print_output();# if this code is executed, there is problem in forking.
exit 0;
